@@ -16,6 +16,7 @@ class ExperimentService(MessageServer):
         self.router.add_route("finish_episode", self.finish_episode)
         self.router.add_route("finish_experiment", self.finish_experiment, FinishExperimentRequest)
         self.router.add_route("get_experiment", self.get_experiment, GetExperimentRequest)
+        self.router.add_route("capture", self.capture, CaptureRequest)
         self.allow_subscription = True
         self.active_experiment = None
         self.active_episode = None
@@ -28,6 +29,13 @@ class ExperimentService(MessageServer):
         self.on_episode_finished = None
         self.on_experiment_finished = None
 
+    def capture(self, request: CaptureRequest):
+        if self.episode_in_progress:
+            self.active_episode.captures.append(request.frame)
+            self.broadcast_subscribed(Message("capture", request))
+            return True
+        return False;
+
     @staticmethod
     def get_experiment_file(experiment_name: str):
         return "logs/" + experiment_name + ".json"
@@ -101,17 +109,24 @@ class ExperimentService(MessageServer):
         return False
 
     def finish_experiment(self, parameters: FinishExperimentRequest) -> bool:
-        self.broadcast_subscribed(Message("experiment_finished", parameters))
-        if self.on_experiment_finished:
-            self.on_experiment_finished(parameters)
-        return True
+        experiment = Experiment.load_from_file(ExperimentService.get_experiment_file(parameters.experiment_name))
+        if experiment:
+            end_time = experiment.start_time + timedelta(minutes=experiment.duration)
+            if end_time > datetime.now():
+                experiment.duration = int((datetime.now() - experiment.start_time).seconds/60)
+                experiment.save(ExperimentService.get_experiment_file(parameters.experiment_name))
+            self.broadcast_subscribed(Message("experiment_finished", parameters))
+            if self.on_experiment_finished:
+                self.on_experiment_finished(parameters)
+            return True
+        return False
 
     @staticmethod
     def get_experiment(parameters: GetExperimentRequest) -> GetExperimentResponse:
         response = GetExperimentResponse()
         experiment = Experiment.load_from_file(ExperimentService.get_experiment_file(parameters.experiment_name))
         if experiment:
-            end_time = experiment.start_time + timedelta(seconds=experiment.duration)
+            end_time = experiment.start_time + timedelta(minutes=experiment.duration)
             if end_time < datetime.now():
                 remaining = 0
             else:
@@ -122,6 +137,7 @@ class ExperimentService(MessageServer):
             response.duration = experiment.duration
             response.remaining_time = remaining
             response.subject_name = experiment.subject_name
+            response.episode_count = len(experiment.episodes)
         return response
 
     def set_tracking_service_ip(self, ip: str):
